!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  b o r e h o l e

!   Purpose    :  Computation of an arbitrary field quantity for a given
!                 borehole position x_pos, y_pos by weighed averaging of the
!                 corresponding gridded 2-d field.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine borehole(field, x_pos, y_pos, dxi, deta, ch_grid, field_val)

use sico_variables

implicit none
double precision, dimension(0:JMAX,0:IMAX), intent(in) :: field
double precision, intent(in) :: x_pos, y_pos, dxi, deta
character (len=*), intent(in) :: ch_grid
double precision, intent(out) :: field_val

integer :: i_kl, i_gr, j_kl, j_gr
double precision :: real_i, real_j, dist1, dist2, dist3, dist4, &
                    weight1, weight2, weight3, weight4

!-------- Neighbour points, weighing factors --------

real_i = (x_pos-xi(0)) /dxi
real_j = (y_pos-eta(0))/deta

i_kl = floor(real_i)
j_kl = floor(real_j)
i_gr = ceiling(real_i)
j_gr = ceiling(real_j)

if (i_kl <= 1) stop ' Subroutine borehole: i_kl <= 1 not allowed!'
if (j_kl <= 1) stop ' Subroutine borehole: j_kl <= 1 not allowed!'
if (i_gr >= IMAX-1) stop ' Subroutine borehole: i_gr >= IMAX-1 not allowed!'
if (j_gr >= JMAX-1) stop ' Subroutine borehole: j_gr >= JMAX-1 not allowed!'

dist1 = sqrt( (x_pos-xi(i_kl))**2 + (y_pos-eta(j_kl))**2 )
dist2 = sqrt( (x_pos-xi(i_kl))**2 + (y_pos-eta(j_gr))**2 )
dist3 = sqrt( (x_pos-xi(i_gr))**2 + (y_pos-eta(j_kl))**2 )
dist4 = sqrt( (x_pos-xi(i_gr))**2 + (y_pos-eta(j_gr))**2 )

weight1 = 1.0d0/(dist1+eps)
weight2 = 1.0d0/(dist2+eps)
weight3 = 1.0d0/(dist3+eps)
weight4 = 1.0d0/(dist4+eps)

!-------- Inverse-distance weighing of the four adjacent grid values --------

if (ch_grid=='grid') then   ! field(j,i) defined on grid points

   field_val = ( weight1*field(j_kl,i_kl) &
                +weight2*field(j_gr,i_kl) &
                +weight3*field(j_kl,i_gr) &
                +weight4*field(j_gr,i_gr) ) &
               /(weight1 + weight2 + weight3 + weight4)

else if (ch_grid=='sg_x') then
                          ! field(j,i) defined on staggered grid in x direction

   field_val = ( weight1*0.5d0*(field(j_kl,i_kl)+field(j_kl,i_kl-1)) &
                +weight2*0.5d0*(field(j_gr,i_kl)+field(j_gr,i_kl-1)) &
                +weight3*0.5d0*(field(j_kl,i_gr)+field(j_kl,i_gr-1)) &
                +weight4*0.5d0*(field(j_gr,i_gr)+field(j_gr,i_gr-1)) ) &
               /(weight1 + weight2 + weight3 + weight4)

else if (ch_grid=='sg_y') then
                          ! field(j,i) defined on staggered grid in y direction

   field_val = ( weight1*0.5d0*(field(j_kl,i_kl)+field(j_kl-1,i_kl)) &
                +weight2*0.5d0*(field(j_gr,i_kl)+field(j_gr-1,i_kl)) &
                +weight3*0.5d0*(field(j_kl,i_gr)+field(j_kl-1,i_gr)) &
                +weight4*0.5d0*(field(j_gr,i_gr)+field(j_gr-1,i_gr)) ) &
               /(weight1 + weight2 + weight3 + weight4)

else

   stop ' Subroutine borehole: Parameter ch_grid has undefined value!'

end if

end subroutine borehole
