!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ e l r a

!   Purpose    :  Computation of the isostatic steady-state displacement of the
!                 lithosphere (wss) for the ELRA model.
!                 Original version by S. Knell, modified by R. Greve.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_elra(z_sl, dxi, deta)

use sico_variables

implicit none

double precision, intent(in) :: z_sl, dxi, deta

integer :: i, j, ir, jr, il, jl, n
integer :: ir_max, jr_max, min_imax_jmax
integer :: il_begin, il_end, jl_begin, jl_end
double precision :: l_r, l_r_inv, rho_g, rho_sw_g, fac_wss
double precision :: ra_max
double precision :: dxi_inv, deta_inv
double precision :: kei_r_incr_inv
double precision, allocatable, dimension(:,:) :: f_0

double precision, parameter :: r_infl = 8.0d0
                               ! Radius of non-locality influence of the elastic
                               ! lithosphere plate (in units of l_r)

!-------- Initialisations --------

l_r = (FLEX_RIG_L/(RHO_A*G))**0.25d0

ra_max = r_infl*l_r   ! Radius of non-locality influence (in m)

l_r_inv  = 1.0d0/l_r
rho_g    = RHO*G
rho_sw_g = RHO_SW*G

fac_wss = l_r**2.0d0/(2.0d0*pi*FLEX_RIG_L)

dxi_inv   = 1.0d0/dxi
deta_inv  = 1.0d0/deta

kei_r_incr_inv = 1.0d0/kei_r_incr

ir_max = floor(ra_max*dxi_inv)
jr_max = floor(ra_max*deta_inv)
         ! Radius of non-locality influence (in grid points)

min_imax_jmax = min(IMAX, JMAX)
ir_max        = min(ir_max, min_imax_jmax)
jr_max        = min(jr_max, min_imax_jmax)
                ! ir_max and jr_max constrained by size of domain

il_begin = -ir_max
il_end   = ir_max+IMAX
jl_begin = -jr_max
jl_end   = jr_max+JMAX

!-------- Ice/water load --------

allocate(f_0(jl_begin:jl_end, il_begin:il_end))

do il=il_begin, il_end
do jl=jl_begin, jl_end

   i = min(max(il, 0), IMAX)
   j = min(max(jl, 0), JMAX)

   if (maske(j,i)==0) then
      f_0(jl,il) = rho_g * area(j,i) * (H_c(j,i) + H_t(j,i))
   else if (maske(j,i)==1) then
      f_0(jl,il) = 0.0d0
   else if (maske(j,i)>=2) then
      f_0(jl,il) = rho_sw_g * area(j,i) * z_sl
                   ! Water load relative to the present sea-level stand (0 m)
                   ! -> can be positive or negative
   end if

end do
end do

!-------- Computation of the steady-state displacement of the lithosphere
!         (wss, positive downward) --------

do i=0, IMAX
do j=0, JMAX

   wss(j,i) = 0.0d0

   do ir=-ir_max, ir_max
   do jr=-jr_max, jr_max

#if GRID==0
      n = nint( dist_dxdy(jr,ir)*l_r_inv*kei_r_incr_inv )
#elif GRID==1
      n = nint( sq_g11_g(j,i)*dist_dxdy(jr,ir)*l_r_inv*kei_r_incr_inv )
              ! The correction with g11 only is OK because g11=g22 for this case
#elif GRID==2
      n = nint( dist_dxdy(jr,ir)*l_r_inv*kei_r_incr_inv )
              ! The distortion due to g11 and g22 is already accounted for in
              ! the variable dist_dxdy(jr,ir)
#endif

      wss(j,i) = wss(j,i) - fac_wss*f_0(jr+j,ir+i)*kei(n)

   end do
   end do

end do
end do

deallocate (f_0)

end subroutine calc_elra
