!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  c a l c _ v x y

!   Purpose    :  Computation of the shear stresses txz, tyz,
!                 the effective shear stress sigma,
!                 the horizontal velocity vx, vy
!                 and the horizontal mass flux qx, qy.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine calc_vxy(avxy3, aqxy1, dzeta_t)

use sico_variables

implicit none
double precision, intent(in) :: avxy3(0:KCMAX), aqxy1(0:KCMAX), dzeta_t

integer :: i, j, kc, kt
double precision :: ctxyz1(0:KCMAX,0:JMAX,0:IMAX), &
                    ctxyz2(0:KTMAX,0:JMAX,0:IMAX)
double precision :: cvxy2(0:KTMAX), cvxy3(0:KCMAX)
double precision :: cqxy0(0:KTMAX), cqxy1(0:KCMAX)
double precision :: vh_max
double precision :: ratefac, ratefac_t, creep

!-------- Computation of stresses --------

!  ------ Term abbreviations

do i=0, IMAX
do j=0, JMAX

if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
!                                ! glaciated land or floating ice

   do kc=0, KCMAX
      ctxyz1(kc,j,i) = RHO*G*H_c(j,i)*(1.0d0-eaz_c_quotient(kc))
   end do

   if (n_cts(j,i) == 1) then   ! temperate layer

      do kt=0, KTMAX
         ctxyz2(kt,j,i) = RHO*G*H_t(j,i)*(1.0d0-zeta_t(kt))
      end do

   else   ! cold base (-1), temperate base (0)

      do kt=0, KTMAX
         ctxyz2(kt,j,i) = 0.0d0
      end do

   end if

else   ! maske(j,i) == (1 or 2)

   do kc=0, KCMAX
      ctxyz1(kc,j,i) = 0.0d0
   end do

   do kt=0, KTMAX
      ctxyz2(kt,j,i) = 0.0d0
   end do

end if

end do
end do

!  ------ Shear stress txz (defined at (i+1/2,j,kc/t))

do i=0, IMAX-1
do j=0, JMAX

   do kc=0, KCMAX
      txz_c(kc,j,i) = -0.5d0*(ctxyz1(kc,j,i)+ctxyz1(kc,j,i+1)) &
                      *dzs_dxi(j,i)
   end do

   do kt=0, KTMAX
      txz_t(kt,j,i) = txz_c(0,j,i) &
                      -0.5d0*(ctxyz2(kt,j,i)+ctxyz2(kt,j,i+1)) &
                      *dzs_dxi(j,i)
   end do

end do
end do

!  ------ Shear stress tyz (defined at (i,j+1/2,kc/t))

do i=0, IMAX
do j=0, JMAX-1

   do kc=0, KCMAX
      tyz_c(kc,j,i) = -0.5d0*(ctxyz1(kc,j,i)+ctxyz1(kc,j+1,i)) &
                      *dzs_deta(j,i)
   end do

   do kt=0, KTMAX
      tyz_t(kt,j,i) = tyz_c(0,j,i) &
                      -0.5d0*(ctxyz2(kt,j,i)+ctxyz2(kt,j+1,i)) &
                      *dzs_deta(j,i)
   end do

end do
end do

!  ------ Effective shear stress sigma

do i=0, IMAX
do j=0, JMAX

   do kc=0, KCMAX
      sigma_c(kc,j,i) = ctxyz1(kc,j,i) &
           *(dzs_dxi_g(j,i)**2+dzs_deta_g(j,i)**2)**0.5d0
   end do

   do kt=0, KTMAX
      sigma_t(kt,j,i) = sigma_c(0,j,i) &
         + ctxyz2(kt,j,i) &
           *(dzs_dxi_g(j,i)**2+dzs_deta_g(j,i)**2)**0.5d0
   end do

end do
end do

!-------- Computation of d_help_c/t
!         (defined on the grid points (i,j,kc/t)) --------

do i=0, IMAX
do j=0, JMAX

   if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
!                                ! glaciated land or floating ice

!  ------ Abbreviations

   do kt=0, KTMAX
      cvxy2(kt) = 2.0d0*H_t(j,i) &
                  *enh_t(kt,j,i) &
                  *ratefac_t(omega_t(kt,j,i)) &
                  *creep(sigma_t(kt,j,i)) &
                  *(ctxyz1(0,j,i)+ctxyz2(kt,j,i)) &
                  *dzeta_t
   end do

   do kc=0, KCMAX
      cvxy3(kc) = 2.0d0*avxy3(kc)*H_c(j,i) &
                  *enh_c(kc,j,i) &
                  *ratefac(temp_c(kc,j,i), temp_c_m(kc,j,i)) &
                  *creep(sigma_c(kc,j,i)) &
                  *ctxyz1(kc,j,i)
   end do

!  ------ d_help_c, d_help_t

   if ( (maske(j,i) == 0).and.(n_cts(j,i) == -1) ) then
                                    ! grounded ice, cold ice base

      do kt=0, KTMAX
         d_help_t(kt,j,i) = d_help_b(j,i)
      end do

      d_help_c(0,j,i) = d_help_t(KTMAX,j,i)

      do kc=0, KCMAX-1
         d_help_c(kc+1,j,i) = d_help_c(kc,j,i) &
                             +0.5d0*(cvxy3(kc+1)+cvxy3(kc))
      end do

   else if ( (maske(j,i) == 3).and.(n_cts(j,i) == -1) ) then
                                    ! floating ice, cold ice base
      do kt=0, KTMAX
         d_help_t(kt,j,i) = d_help_b(j,i)
      end do

      d_help_c(0,j,i) = d_help_t(KTMAX,j,i)

      do kc=0, KCMAX-1
         d_help_c(kc+1,j,i) = d_help_c(kc,j,i) &
                             +0.5d0*(cvxy3(kc+1)+cvxy3(kc))
      end do

   else if (n_cts(j,i) == 0) then   ! temperate ice base

      do kt=0, KTMAX
         d_help_t(kt,j,i) = d_help_b(j,i)
      end do

      d_help_c(0,j,i) = d_help_t(KTMAX,j,i)

      do kc=0, KCMAX-1
         d_help_c(kc+1,j,i) = d_help_c(kc,j,i) &
                             +0.5d0*(cvxy3(kc+1)+cvxy3(kc))
      end do

   else   ! n_cts(j,i) == 1, temperate ice layer

      d_help_t(0,j,i) = d_help_b(j,i)

      do kt=0, KTMAX-1
         d_help_t(kt+1,j,i) = d_help_t(kt,j,i) &
                             +0.5d0*(cvxy2(kt+1)+cvxy2(kt))
      end do

      d_help_c(0,j,i) = d_help_t(KTMAX,j,i)

      do kc=0, KCMAX-1
         d_help_c(kc+1,j,i) = d_help_c(kc,j,i) &
                             +0.5d0*(cvxy3(kc+1)+cvxy3(kc))
      end do

   end if

else   ! maske(j,i) == (1 oder 2)

   do kt=0, KTMAX
      d_help_t(kt,j,i) = 0.0d0
   end do

   do kc=0, KCMAX
      d_help_c(kc,j,i) = 0.0d0
   end do

end if

end do
end do

!-------- Computation of vx_c/t (defined at (i+1/2,j,kc/t)) --------

do i=0, IMAX-1
do j=1, JMAX-1

   do kt=0, KTMAX
      vx_t(kt,j,i) = -0.5d0*(d_help_t(kt,j,i)+d_help_t(kt,j,i+1)) &
                     *dzs_dxi(j,i)
   end do

   do kc=0, KCMAX
      vx_c(kc,j,i) = -0.5d0*(d_help_c(kc,j,i)+d_help_c(kc,j,i+1)) &
                     *dzs_dxi(j,i)
   end do

end do
end do

!-------- Computation of vy_c/t (defined at (i,j+1/2,kc/t)) --------

do i=1, IMAX-1
do j=0, JMAX-1

   do kt=0, KTMAX
      vy_t(kt,j,i) = -0.5d0*(d_help_t(kt,j,i)+d_help_t(kt,j+1,i)) &
                     *dzs_deta(j,i)
   end do

   do kc=0, KCMAX
      vy_c(kc,j,i) = -0.5d0*(d_help_c(kc,j,i)+d_help_c(kc,j+1,i)) &
                     *dzs_deta(j,i)
   end do

end do
end do

!-------- Limitation of computed vx_c/t, vy_c/t to the interval
!         [-VH_MAX, VH_MAX] --------

vh_max = VH_MAX/YEAR_SEC

do i=0, IMAX
do j=0, JMAX
   do kt=0, KTMAX
      vx_t(kt,j,i) = max(vx_t(kt,j,i), -vh_max)
      vx_t(kt,j,i) = min(vx_t(kt,j,i),  vh_max)
      vy_t(kt,j,i) = max(vy_t(kt,j,i), -vh_max)
      vy_t(kt,j,i) = min(vy_t(kt,j,i),  vh_max)
   end do
   do kc=0, KCMAX
      vx_c(kc,j,i) = max(vx_c(kc,j,i), -vh_max)
      vx_c(kc,j,i) = min(vx_c(kc,j,i),  vh_max)
      vy_c(kc,j,i) = max(vy_c(kc,j,i), -vh_max)
      vy_c(kc,j,i) = min(vy_c(kc,j,i),  vh_max)
   end do
end do
end do

!-------- Computation of h_diff
!         (defined on the grid points (i,j)) --------

do i=0, IMAX
do j=0, JMAX

if ( (maske(j,i) == 0).or.(maske(j,i) == 3) ) then
!                                ! glaciated land or floating ice

!  ------ Abbreviations

   do kt=0, KTMAX
      cqxy0(kt) = H_t(j,i)*d_help_t(kt,j,i)*dzeta_t
   end do

   do kc=0, KCMAX
      cqxy1(kc) = aqxy1(kc)*H_c(j,i)*d_help_c(kc,j,i)
   end do

!  ------ h_diff

   h_diff(j,i) = 0.0d0

   if (n_cts(j,i) == 1) then

      do kt=0, KTMAX-1
         h_diff(j,i) = h_diff(j,i)+0.5d0*(cqxy0(kt+1)+cqxy0(kt))
      end do

   end if

   do kc=0, KCMAX-1
      h_diff(j,i) = h_diff(j,i)+0.5d0*(cqxy1(kc+1)+cqxy1(kc))
   end do

!  ------ Limitation of h_diff

   if (h_diff(j,i) < HD_MIN) h_diff(j,i) = 0.0d0
   if (h_diff(j,i) > HD_MAX) h_diff(j,i) = HD_MAX

else   ! maske(j,i) == (1 oder 2)

   h_diff(j,i) = 0.0d0

end if

end do
end do

!-------- Computation of the horizontal mass flux --------

do i=0, IMAX-1
do j=0, JMAX
   qx(j,i) = -0.5d0*(h_diff(j,i)+h_diff(j,i+1))*dzs_dxi(j,i)
end do
end do

do i=0, IMAX
do j=0, JMAX-1
   qy(j,i) = -0.5d0*(h_diff(j,i)+h_diff(j+1,i))*dzs_deta(j,i)
end do
end do

end subroutine calc_vxy
