!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  p d d

!   Purpose    :  Computation of the positive degree days (PDD) with
!                 statistical temperature fluctuations;
!                 based on semi-analytical solution by Reinhard Calov.
!                 This subroutine uses years as time unit (as opposed to
!                 the seconds which are usually used by SICOPOLIS)!

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine pdd(temp_mm, s_stat, ET)

implicit none
double precision, dimension(12), intent(in) :: temp_mm
double precision, intent(in) :: s_stat
double precision, intent(out) :: ET

integer :: n
double precision :: inv_sqrt2pi, inv_s_stat, inv_sqrt2
double precision :: pdd_sum

double precision, parameter :: pi=3.141592653589793d0
double precision, parameter :: time_year     = 1.0d0, &      ! period 1 year
                               time_year_inv = 1.0d0/time_year, &
                               d_time        = 1.0d0/12.0d0  ! time-step 1 month

inv_sqrt2pi = 1.0d0/sqrt(2.0d0*pi)
inv_s_stat  = 1.0d0/s_stat
inv_sqrt2   = 1.0d0/sqrt(2.0d0)

pdd_sum = 0.0d0

do n=1, 12   ! month counter
   pdd_sum = pdd_sum &
             + ( s_stat*inv_sqrt2pi*exp(-0.5d0*(temp_mm(n)*inv_s_stat)**2) &
                 + 0.5d0*temp_mm(n)*erfcc(-temp_mm(n)*inv_s_stat*inv_sqrt2) ) &
               *d_time   ! positive degree days (in a * deg C)
end do

ET   = pdd_sum*time_year_inv   ! temperature excess   (in deg C)

contains

  double precision function erfcc(x)

  implicit none
  double precision, intent(in) :: x

  ! Returns the complementary error function erfc(x) with fractional error
  ! everywhere less than 1.2 x 10^(-7);
  ! see Press et al., 'Numerical Recipes in Fortran 77'.

  double precision :: t, z

  z = abs(x)
  t = 1.0d0/(1.0d0+0.5d0*z)

  erfcc = t*exp(-z*z-1.26551223d0+t*(1.00002368d0+t*(0.37409196d0+ &
          t*(0.09678418d0+t*(-0.18628806d0+t*(0.27886807d0+t*(-1.13520398d0+ &
          t*(1.48851587d0+t*(-0.82215223d0+t*0.17087277d0)))))))))

  if (x < 0.0d0) erfcc = 2.0d0-erfcc

  end function erfcc

end subroutine pdd
