!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  s o r _ s p r s

!   Purpose    :  SOR solver for a system of linear equations a*x=rs
!                 [matrix a in row-indexed compact storage, represented
!                 by arrays sa (values) and ija (indices);
!                 see Press et al., 'Numerical Recipes in Fortran 77'].

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine sor_sprs(sa, ija, rs, n_sprs, n, omega, x, eps_sor, ier)

implicit none
integer :: it, i, n, n_sprs
integer :: ier
integer, dimension(n_sprs) :: ija
double precision, dimension(n_sprs) :: sa
double precision, dimension(n) :: rs, x
double precision, allocatable, dimension(:) :: xnu
double precision :: omega, eps_sor
double precision :: b_i
logical flag_convergence

allocate(xnu(n))

iter_loop : do it=1, 1000

   xnu = x

   do i=1, n
      call sprsax_i(sa, ija, x, b_i, n, i)
      x(i) = x(i)-omega*(b_i-rs(i))/sa(i)
   end do

   flag_convergence = .true.
   do i=1, n
      if (abs(x(i)-xnu(i)) > eps_sor) then
         flag_convergence = .false.
         exit
      end if
   end do

   if (flag_convergence) then
      write(6,'(11x,a,i5)') 'it = ', it
      ier = 0   ! convergence criterion fulfilled
      deallocate(xnu)
      return
   end if

end do iter_loop

write(6,'(a,i5)') 'it = ', it
ier = -1   ! convergence criterion not fulfilled
deallocate(xnu)

end subroutine sor_sprs

!----------------------------------------------------------------------

subroutine sprsax_i(sa, ija, x, b_i, n, i)

implicit none
integer :: n, ija(*)
double precision :: sa(*), x(n), b_i

! Multiply a matrix in row-indexed compact storage arrays sa and ija by
! a vector x(1:n), giving the i-component b_i of the resulting vector.

integer i, k

if (ija(1) /= n+2) stop 'Mismatched vector and matrix in sprsax!'

b_i=sa(i)*x(i)   ! Start with diagonal term.

do k=ija(i),ija(i+1)-1   ! Loop over off-diagonal terms.
   b_i=b_i+sa(k)*x(ija(k))
end do

end subroutine sprsax_i
