!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  t o p o g r a d _ 1

!   Purpose    :  Calculation of topography gradients on the
!                 staggered grid and on the grid points
!                 (the latter by second-order discretization).
!                 Length rescaling with the corresponding components
!                 of the metric tensor already included.

!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine topograd_1(dxi, deta, n_switch)

use sico_variables

implicit none
integer i, j
integer n_switch
double precision dxi, deta, dxi_inv, deta_inv
double precision zs_help(0:JMAX,0:IMAX), zm_help(0:JMAX,0:IMAX), &
       zb_help(0:JMAX,0:IMAX)

dxi_inv  = 1.0d0/dxi
deta_inv = 1.0d0/deta

!-------- Distinguish between old and new topography data --------

if (n_switch.eq.1) then

   do i=0, IMAX
   do j=0, JMAX
      zs_help(j,i) = zs(j,i)
      zm_help(j,i) = zm(j,i)
      zb_help(j,i) = zb(j,i)
   end do
   end do

else if (n_switch.eq.2) then

   do i=0, IMAX
   do j=0, JMAX
      zs_help(j,i) = zs_neu(j,i)
      zm_help(j,i) = zm_neu(j,i)
      zb_help(j,i) = zb_neu(j,i)
   end do
   end do

else
   stop ' Wrong value for n_switch!'
end if

!-------- Topography gradients on the staggered grid --------

!  ------ x-derivatives

do i=0, IMAX-1
do j=0, JMAX
   dzs_dxi(j,i)  = (zs_help(j,i+1)-zs_help(j,i))*dxi_inv &
                   *insq_g11_sgx(j,i)
   dzm_dxi(j,i)  = (zm_help(j,i+1)-zm_help(j,i))*dxi_inv &
                   *insq_g11_sgx(j,i)
   dzb_dxi(j,i)  = (zb_help(j,i+1)-zb_help(j,i))*dxi_inv &
                   *insq_g11_sgx(j,i)
   dH_c_dxi(j,i) = dzs_dxi(j,i)-dzm_dxi(j,i)
   dH_t_dxi(j,i) = dzm_dxi(j,i)-dzb_dxi(j,i)
end do
end do

!  ------ y-derivatives

do i=0, IMAX
do j=0, JMAX-1
   dzs_deta(j,i)  = (zs_help(j+1,i)-zs_help(j,i))*deta_inv &
                   *insq_g22_sgy(j,i)
   dzm_deta(j,i)  = (zm_help(j+1,i)-zm_help(j,i))*deta_inv &
                   *insq_g22_sgy(j,i)
   dzb_deta(j,i)  = (zb_help(j+1,i)-zb_help(j,i))*deta_inv &
                   *insq_g22_sgy(j,i)
   dH_c_deta(j,i) = dzs_deta(j,i)-dzm_deta(j,i)
   dH_t_deta(j,i) = dzm_deta(j,i)-dzb_deta(j,i)
end do
end do

!-------- Topography gradients on the grid points --------

!  ------ x-derivatives

do i=1, IMAX-1
do j=0, JMAX
   dzs_dxi_g(j,i)  = (zs_help(j,i+1)-zs_help(j,i-1))*0.5d0*dxi_inv &
                     *insq_g11_g(j,i)
   dzm_dxi_g(j,i)  = (zm_help(j,i+1)-zm_help(j,i-1))*0.5d0*dxi_inv &
                     *insq_g11_g(j,i)
   dzb_dxi_g(j,i)  = (zb_help(j,i+1)-zb_help(j,i-1))*0.5d0*dxi_inv &
                     *insq_g11_g(j,i)
   dH_c_dxi_g(j,i) = dzs_dxi_g(j,i)-dzm_dxi_g(j,i)
   dH_t_dxi_g(j,i) = dzm_dxi_g(j,i)-dzb_dxi_g(j,i)
end do
end do

do j=0, JMAX
   dzs_dxi_g(j,0)     = (zs_help(j,1)-zs_help(j,0))*dxi_inv &
                        *insq_g11_g(j,0)
   dzm_dxi_g(j,0)     = (zm_help(j,1)-zm_help(j,0))*dxi_inv &
                        *insq_g11_g(j,0)
   dzb_dxi_g(j,0)     = (zb_help(j,1)-zb_help(j,0))*dxi_inv &
                        *insq_g11_g(j,0)
   dH_c_dxi_g(j,0)    = dzs_dxi_g(j,0)-dzm_dxi_g(j,0)
   dH_t_dxi_g(j,0)    = dzm_dxi_g(j,0)-dzb_dxi_g(j,0)
   dzs_dxi_g(j,IMAX)  = (zs_help(j,IMAX)-zs_help(j,IMAX-1)) &
                        *dxi_inv &
                        *insq_g11_g(j,IMAX)
   dzm_dxi_g(j,IMAX)  = (zm_help(j,IMAX)-zm_help(j,IMAX-1)) &
                        *dxi_inv &
                        *insq_g11_g(j,IMAX)
   dzb_dxi_g(j,IMAX)  = (zb_help(j,IMAX)-zb_help(j,IMAX-1)) &
                        *dxi_inv &
                        *insq_g11_g(j,IMAX)
   dH_c_dxi_g(j,IMAX) = dzs_dxi_g(j,IMAX)-dzm_dxi_g(j,IMAX)
   dH_t_dxi_g(j,IMAX) = dzm_dxi_g(j,IMAX)-dzb_dxi_g(j,IMAX)
end do

!  ------ y-derivatives

do i=0, IMAX
do j=1, JMAX-1
   dzs_deta_g(j,i)  = (zs_help(j+1,i)-zs_help(j-1,i)) &
                      *0.5d0*deta_inv &
                      *insq_g22_g(j,i)
   dzm_deta_g(j,i)  = (zm_help(j+1,i)-zm_help(j-1,i)) &
                      *0.5d0*deta_inv &
                      *insq_g22_g(j,i)
   dzb_deta_g(j,i)  = (zb_help(j+1,i)-zb_help(j-1,i)) &
                      *0.5d0*deta_inv &
                      *insq_g22_g(j,i)
   dH_c_deta_g(j,i) = dzs_deta_g(j,i)-dzm_deta_g(j,i)
   dH_t_deta_g(j,i) = dzm_deta_g(j,i)-dzb_deta_g(j,i)
end do
end do

do i=0, IMAX
   dzs_deta_g(0,i)     = (zs_help(1,i)-zs_help(0,i))*deta_inv &
                         *insq_g22_g(0,i)
   dzm_deta_g(0,i)     = (zm_help(1,i)-zm_help(0,i))*deta_inv &
                         *insq_g22_g(0,i)
   dzb_deta_g(0,i)     = (zb_help(1,i)-zb_help(0,i))*deta_inv &
                         *insq_g22_g(0,i)
   dH_c_deta_g(0,i)    = dzs_deta_g(0,i)-dzm_deta_g(0,i)
   dH_t_deta_g(0,i)    = dzm_deta_g(0,i)-dzb_deta_g(0,i)
   dzs_deta_g(JMAX,i)  = (zs_help(JMAX,i)-zs_help(JMAX-1,i)) &
                         *deta_inv &
                         *insq_g22_g(JMAX,i)
   dzm_deta_g(JMAX,i)  = (zm_help(JMAX,i)-zm_help(JMAX-1,i)) &
                         *deta_inv &
                         *insq_g22_g(JMAX,i)
   dzb_deta_g(JMAX,i)  = (zb_help(JMAX,i)-zb_help(JMAX-1,i)) &
                         *deta_inv &
                         *insq_g22_g(JMAX,i)
   dH_c_deta_g(JMAX,i) = dzs_deta_g(JMAX,i)-dzm_deta_g(JMAX,i)
   dH_t_deta_g(JMAX,i) = dzm_deta_g(JMAX,i)-dzb_deta_g(JMAX,i)
end do

return
end
