!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Subroutine :  b o u n d a r y

!   Purpose    :  Definition of the surface temperature (must be less
!                 than 0 deg C!) and of the accumulation-ablation
!                 function.

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

subroutine boundary(time, dxi, deta, delta_ts, glac_index, griptemp, &
              specmap_zsl, z_sl, z_mar, &
              grip_time_min, grip_time_stp, grip_time_max, &
              specmap_time_min, specmap_time_stp, specmap_time_max, &
              z_sle_forcing, time_forcing, ndata_forcing)

use sico_variables

implicit none
integer i, j, n
integer mask_update
integer i_gr, i_kl
integer ndata_forcing
integer grip_time_min, grip_time_stp, grip_time_max, &
        specmap_time_min, specmap_time_stp, specmap_time_max
integer ndata_grip, ndata_specmap
double precision time, delta_ts, glac_index
double precision dxi, deta
double precision z_sl, z_sl_min, t1, t2, t3, t4, t5, t6, z_mar
double precision griptemp(0:10000), specmap_zsl(0:10000), time_gr, time_kl
double precision z_sle_present, z_sle_help, &
       z_sle_forcing(0:10000), time_forcing(0:10000)
double precision precip(0:JMAX,0:IMAX,0:12), &
       snowfall(0:JMAX,0:IMAX), rainfall(0:JMAX,0:IMAX), &
       melt(0:JMAX,0:IMAX), melt_star(0:JMAX,0:IMAX)
double precision temp_ma(0:JMAX,0:IMAX), temp_mm(0:JMAX,0:IMAX,12)
double precision, dimension(12) :: temp_mm_help
double precision :: temp_jja_help
double precision ET(0:JMAX,0:IMAX)
double precision gamma_t, temp_diff
double precision gamma_p, zs_thresh, &
       temp_rain, temp_snow, inv_delta_temp_rain_snow, &
       precip_fact, frac_solid
double precision s_stat, beta1, beta2, Pmax, mu, lambda_lti, temp_lti
double precision, parameter :: &
       inv_twelve = 1.0d0/12.0d0, one_third = 1.0d0/3.0d0
logical check_point(0:JMAX,0:IMAX)

!-------- Surface-temperature deviation from present values --------

#if TSURFACE==1
delta_ts = DELTA_TS0
!                           ! Steady state with prescribed constant
!                           ! air-temperature deviation
#elif TSURFACE==3
delta_ts = SINE_AMPLIT &
           *cos(2.0d0*pi*time/(SINE_PERIOD*YEAR_SEC)) &
           -SINE_AMPLIT
!                           ! Sinusoidal air-temperature forcing
#elif TSURFACE==4

!  ------ delta_ts from ice-core record

ndata_grip = (grip_time_max-grip_time_min)/grip_time_stp

if (time/YEAR_SEC.lt.dble(grip_time_min)) then
   delta_ts = griptemp(0)
else if (time/YEAR_SEC.lt.dble(grip_time_max)) then

   i_kl = floor(((time/YEAR_SEC)-dble(grip_time_min))/dble(grip_time_stp))
   i_kl = max(i_kl, 0)

   i_gr = ceiling(((time/YEAR_SEC)-dble(grip_time_min))/dble(grip_time_stp))
   i_gr = min(i_gr, ndata_grip)

   if (i_kl.eq.i_gr) then

      delta_ts = griptemp(i_kl)

   else

      time_kl = (grip_time_min + i_kl*grip_time_stp) *YEAR_SEC
      time_gr = (grip_time_min + i_gr*grip_time_stp) *YEAR_SEC

      delta_ts = griptemp(i_kl) &
                +(griptemp(i_gr)-griptemp(i_kl)) &
                *(time-time_kl)/(time_gr-time_kl)
                 ! linear interpolation of the ice-core data

   end if

else
   delta_ts  = griptemp(ndata_grip)
end if

delta_ts = delta_ts * GRIP_TEMP_FACT
!                ! modification by constant factor

!-------- Glacial index --------

#elif TSURFACE==5

if (time/YEAR_SEC < dble(gi_time_min)) then
   glac_index = glacial_index(0)
else if (time/YEAR_SEC < dble(gi_time_max)) then

   i_kl = floor(((time/YEAR_SEC)-dble(gi_time_min))/dble(gi_time_stp))
   i_kl = max(i_kl, 0)

   i_gr = ceiling(((time/YEAR_SEC)-dble(gi_time_min))/dble(gi_time_stp))
   i_gr = min(i_gr, ndata_gi)

   if (i_kl == i_gr) then

      glac_index = glacial_index(i_kl)

   else

      time_kl = (gi_time_min + i_kl*gi_time_stp) *YEAR_SEC
      time_gr = (gi_time_min + i_gr*gi_time_stp) *YEAR_SEC

      glac_index = glacial_index(i_kl) &
                +(glacial_index(i_gr)-glacial_index(i_kl)) &
                *(time-time_kl)/(time_gr-time_kl)
                 ! linear interpolation of the glacial-index data

   end if

else
   glac_index  = glacial_index(ndata_gi)
end if

#endif

!-------- Sea level --------

#if SEA_LEVEL==1
!  ------ constant sea level
z_sl = Z_SL0

#elif SEA_LEVEL==2
!  ------ saw-tooth-shaped palaeoclimatic sea-level forcing

z_sl_min = -130.0d0

t1 = -250000.0d0 *YEAR_SEC
t2 = -140000.0d0 *YEAR_SEC
t3 = -125000.0d0 *YEAR_SEC
t4 =  -21000.0d0 *YEAR_SEC
t5 =   -8000.0d0 *YEAR_SEC
t6 =       0.0d0 *YEAR_SEC

if (time.lt.t1) then
   z_sl = 0.0d0
else if (time.lt.t2) then
   z_sl = z_sl_min*(time-t1)/(t2-t1)
else if (time.lt.t3) then
   z_sl = -z_sl_min*(time-t3)/(t3-t2)
else if (time.lt.t4) then
   z_sl = z_sl_min*(time-t3)/(t4-t3)
else if (time.lt.t5) then
   z_sl = -z_sl_min*(time-t5)/(t5-t4)
else if (time.lt.t6) then
   z_sl = 0.0d0
else
   z_sl = 0.0d0
end if

#elif SEA_LEVEL==3

!  ------ z_sl from the SPECMAP record

ndata_specmap = (specmap_time_max-specmap_time_min)/specmap_time_stp

if (time/YEAR_SEC.lt.dble(specmap_time_min)) then
   z_sl = specmap_zsl(0)
else if (time/YEAR_SEC.lt.dble(specmap_time_max)) then

   i_kl = floor(((time/YEAR_SEC)-dble(specmap_time_min))/dble(specmap_time_stp))
   i_kl = max(i_kl, 0)

   i_gr = ceiling(((time/YEAR_SEC)-dble(specmap_time_min))/dble(specmap_time_stp))
   i_gr = min(i_gr, ndata_specmap)

   if (i_kl.eq.i_gr) then

      z_sl = specmap_zsl(i_kl)

   else

      time_kl = (specmap_time_min + i_kl*specmap_time_stp) *YEAR_SEC
      time_gr = (specmap_time_min + i_gr*specmap_time_stp) *YEAR_SEC

      z_sl = specmap_zsl(i_kl) &
                +(specmap_zsl(i_gr)-specmap_zsl(i_kl)) &
                *(time-time_kl)/(time_gr-time_kl)
                 ! linear interpolation of the sea-level data

   end if

else
   z_sl  = specmap_zsl(ndata_specmap)
end if

#elif SEA_LEVEL==4

!  ------ z_sl from previously computed sea-level-equivalent scenario

z_sle_present = 7.0d0   ! sea-level equivalent of present
                        ! northern hemisphere ice sheets, in m

if (time/YEAR_SEC.lt.time_forcing(0)) then
   z_sle_help = z_sle_forcing(0)
else if (time/YEAR_SEC.lt.time_forcing(ndata_forcing)) then

   i_kl = int( (time/YEAR_SEC-time_forcing(0)) &
               /(time_forcing(1)-time_forcing(0)) )
   i_gr = i_kl+1

   if (i_kl.lt.0) then
      i_kl = 0
      i_gr = 1
   else if (i_gr.gt.ndata_forcing) then
      i_gr = ndata_forcing
      i_kl = ndata_forcing-1
   end if

   time_kl = time_forcing(i_kl)*YEAR_SEC
   time_gr = time_forcing(i_gr)*YEAR_SEC

   z_sle_help = z_sle_forcing(i_kl) &
               +(z_sle_forcing(i_gr)-z_sle_forcing(i_kl)) &
               *(time-time_kl)/(time_gr-time_kl)
!                  ! linear interpolation from the computed z_sle
else
   z_sle_help = z_sle_forcing(ndata_forcing)
end if

z_sl = z_sle_present - Z_SLE_FACT * z_sle_help

#endif

!  ------ Minimum bedrock elevation for extent of marine ice

#if MARGIN==2
z_mar = Z_MAR
#elif MARGIN==3
z_mar = FACT_Z_MAR*z_sl
#elif MARGIN==4
if (z_sl >= -80.0d0) then
   z_mar = 2.5d0*z_sl
else
   z_mar = 10.25d0*(z_sl+80.0d0)-200.0d0
end if
z_mar = FACT_Z_MAR*z_mar
#endif

!  ------ Update of the mask according to the sea level

!    ---- Check all sea and floating-ice points and their direct
!         neighbours

do i=0, IMAX
do j=0, JMAX
   check_point(j,i) = .false.
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if (maske(j,i).ge.2) then
      check_point(j  ,i  ) = .true.
      check_point(j  ,i+1) = .true.
      check_point(j  ,i-1) = .true.
      check_point(j+1,i  ) = .true.
      check_point(j-1,i  ) = .true.
   end if
end do
end do

do i=1, IMAX-1
do j=1, JMAX-1
   if (check_point(j,i)) then
      maske_neu(j,i) = mask_update(z_sl, i, j)
   end if
end do
end do

!    ---- Assign new values of the mask

do i=1, IMAX-1
do j=1, JMAX-1
   if (check_point(j,i)) then
      maske(j,i) = maske_neu(j,i)
   end if
end do
end do

!-------- Surface air temperatures --------

gamma_t = -6.5d-03   ! atmospheric lapse rate

do i=0, IMAX
do j=0, JMAX

#if (TSURFACE <= 4)

!  ------ Correction of present monthly temperatures with elevation changes
!         and temperature deviation delta_ts

   temp_diff = gamma_t*(zs(j,i)-zs_ref(j,i)) + delta_ts

   do n=1, 12   ! month counter
      temp_mm(j,i,n) = temp_mm_present(j,i,n) + temp_diff
   end do

#elif (TSURFACE == 5)

!  ------ Correction of present monthly temperatures with LGM anomaly and
!         glacial index as well as elevation changes

   temp_diff = gamma_t*(zs(j,i)-zs_ref(j,i))

   do n=1, 12   ! month counter
      temp_mm(j,i,n) = temp_mm_present(j,i,n) &
                       + glac_index*temp_mm_lgm_anom(j,i,n) &
                       + temp_diff
   end do

#endif

!  ------ Mean annual air temperature

   temp_ma(j,i) = 0.0d0   ! initialisation value

   do n=1, 12   ! month counter
      temp_ma(j,i) = temp_ma(j,i) + temp_mm(j,i,n)*inv_twelve
   end do

end do
end do

!-------- Accumulation-ablation function as_perp --------

gamma_p   = -0.001d0*log(2.0d0)   ! Precipitation lapse rate,
                                  ! in m^(-1)
zs_thresh = 2000.0d0   ! Threshold for elevation desertification, in m
temp_rain =    7.0d0   ! Threshold temperature for precipitation = 100% rain, &
                       ! in deg C
temp_snow =  -10.0d0   ! Threshold temperature for precipitation = 100% snow, &
                       ! in deg C

inv_delta_temp_rain_snow = 1.0d0/(temp_rain-temp_snow)

#if (ABLSURFACE==1 || ABLSURFACE==2)

s_stat = S_STAT_0
beta1  = BETA1_0  *(0.001d0/86400.0d0)*(RHO_W/RHO)
                        ! (mm WE)/(d*deg C) --> (m IE)/(s*deg C)
beta2  = BETA2_0  *(0.001d0/86400.0d0)*(RHO_W/RHO)
                        ! (mm WE)/(d*deg C) --> (m IE)/(s*deg C)
Pmax   = PMAX_0
mu     = MU_0     *(1000.0d0*86400.0d0)*(RHO/RHO_W)
                        ! (d*deg C)/(mm WE) --> (s*deg C)/(m IE)

#elif (ABLSURFACE==3)

lambda_lti = LAMBDA_LTI  *(0.001d0/YEAR_SEC)*(RHO_W/RHO)
                         ! (mm WE)/(a*deg C) --> (m IE)/(s*deg C)
temp_lti   = TEMP_LTI
mu         = 0.0d0       ! no superimposed ice considered

#endif

do i=0, IMAX
do j=0, JMAX

   if (maske(j,i).le.1) then   ! Land point !

!  ------ Accumulation

#if (ACCSURFACE <= 4)

!    ---- Elevation desertification of precipitation

      if (zs_ref(j,i) < zs_thresh) then
         precip_fact &
            = exp(gamma_p*(max(zs(j,i),zs_thresh)-zs_thresh))
      else
         precip_fact &
            = exp(gamma_p*(max(zs(j,i),zs_thresh)-zs_ref(j,i)))
      end if

      do n=1, 12   ! month counter
         precip(j,i,n) = precip_present(j,i,n)*precip_fact
      end do

#endif

!    ---- Precipitation change related to changing climate

#if ACCSURFACE==1
      precip_fact = ACCFACT
#elif ACCSURFACE==2
      precip_fact = 1.0d0 + GAMMA_S*delta_ts
#elif ACCSURFACE==3
      precip_fact = exp(GAMMA_S*delta_ts)
#endif

#if (ACCSURFACE <= 4)

      precip(j,i,0) = 0.0d0   ! initialisation value for mean annual precip

      do n=1, 12   ! month counter
         precip(j,i,n) = precip(j,i,n)*precip_fact   ! monthly precip
         precip(j,i,0) = precip(j,i,0) + precip(j,i,n)*inv_twelve
                                                     ! mean annual precip
      end do

#elif (ACCSURFACE == 5)

      precip(j,i,0) = 0.0d0   ! initialisation value for mean annual precip

      do n=1, 12   ! month counter

#if (PRECIP_ANOM_INTERPOL==1)
         precip_fact = 1.0d0-glac_index+glac_index*precip_lgm_anom(j,i,n)
                       ! interpolation with a linear function
#elif (PRECIP_ANOM_INTERPOL==2)
         precip_fact = exp(-glac_index*gamma_precip_lgm_anom(j,i,n))
                       ! interpolation with an exponential function
#endif

         precip(j,i,n) = precip_present(j,i,n)*precip_fact   ! monthly precip
         precip(j,i,0) = precip(j,i,0) + precip(j,i,n)*inv_twelve
                                                         ! mean annual precip
      end do

#endif

!    ---- Annual accumulation, snowfall and rainfall rates

      accum(j,i) = precip(j,i,0)

      snowfall(j,i) = 0.0d0   ! initialisation value

      do n=1, 12   ! month counter

         if (temp_mm(j,i,n) >= temp_rain) then
            frac_solid = 0.0d0
         else if (temp_mm(j,i,n) <= temp_snow) then
            frac_solid = 1.0d0
         else
            frac_solid = (temp_rain-temp_mm(j,i,n))*inv_delta_temp_rain_snow
         end if

         snowfall(j,i) = snowfall(j,i) + precip(j,i,n)*frac_solid*inv_twelve

      end do

      rainfall(j,i) = precip(j,i,0) - snowfall(j,i)

      if (snowfall(j,i) < -eps) stop ' Negative snowfall rate!'
      if (rainfall(j,i) < -eps) stop ' Negative rainfall rate!'

!  ------ Ablation

!    ---- Runoff

#if (ABLSURFACE==1 || ABLSURFACE==2)

!      -- Temperature excess ET

      do n=1, 12   ! month counter
         temp_mm_help(n) = temp_mm(j,i,n)
      end do

      call pdd(temp_mm_help, s_stat, ET(j,i))

!      -- Formation rate of superimposed ice (melt_star), melt rate (melt)
!         and runoff rate (runoff)

#if (ABLSURFACE==1)

      if ((beta1*ET(j,i)) <= (Pmax*snowfall(j,i))) then
         melt_star(j,i) = beta1*ET(j,i)
         melt(j,i)      = 0.0d0
         runoff(j,i)    = melt(j,i)+rainfall(j,i)
      else
         melt_star(j,i) = Pmax*snowfall(j,i)
         melt(j,i)      = beta2*(ET(j,i)-melt_star(j,i)/beta1)
         runoff(j,i)    = melt(j,i)+rainfall(j,i)
      end if

#elif (ABLSURFACE==2)

      if ( rainfall(j,i) <= (Pmax*snowfall(j,i)) ) then

         if ( (rainfall(j,i)+beta1*ET(j,i)) <= (Pmax*snowfall(j,i)) ) then
            melt_star(j,i) = rainfall(j,i)+beta1*ET(j,i)
            melt(j,i)      = 0.0d0
            runoff(j,i)    = melt(j,i)
         else
            melt_star(j,i) = Pmax*snowfall(j,i)
            melt(j,i)      = beta2 &
                             *(ET(j,i)-(melt_star(j,i)-rainfall(j,i))/beta1)
            runoff(j,i)    = melt(j,i)
         end if

      else

         melt_star(j,i) = Pmax*snowfall(j,i)
         melt(j,i)      = beta2*ET(j,i)
         runoff(j,i)    = melt(j,i) + rainfall(j,i)-Pmax*snowfall(j,i)

      end if

#endif

#elif (ABLSURFACE==3)

      temp_jja_help  = one_third*(temp_mm(j,i,6)+temp_mm(j,i,7)+temp_mm(j,i,8))

      melt_star(j,i) = 0.0d0   ! no superimposed ice considered
      melt(j,i)      = lambda_lti*max((temp_jja_help-temp_lti), 0.0d0)
      runoff(j,i)    = melt(j,i) + rainfall(j,i)

#endif

!    ---- Evaporation

      evap(j,i) = 0.0d0

!  ------ Accumulation minus ablation

      as_perp(j,i) = accum(j,i) - evap(j,i) - runoff(j,i)

!  ------ Ice-surface temperature (10-m firn temperature) temp_s,
!         including empirical firn-warming correction due to
!         refreezing meltwater when superimposed ice is formed

      if (melt_star(j,i).ge.melt(j,i)) then
         temp_s(j,i) = temp_ma(j,i) &
                       +mu*(melt_star(j,i)-melt(j,i))
      else
         temp_s(j,i) = temp_ma(j,i)
      end if

      if (temp_s(j,i) > -0.001d0) temp_s(j,i) = -0.001d0
                                  ! Cut-off of positive air temperatures

   else   ! maske(j,i).ge.2, sea or floating ice !

!  ------ Accumulation

      precip(j,i,0) = 0.0d0
      accum(j,i)    = 0.0d0
      snowfall(j,i) = 0.0d0
      rainfall(j,i) = 0.0d0

!  ------ Ablation

!    ---- Runoff

#if (ABLSURFACE==1 || ABLSURFACE==2)

!      -- Temperature excess ET

      do n=1, 12   ! month counter
         temp_mm_help(n) = temp_mm(j,i,n)
      end do

      call pdd(temp_mm_help, s_stat, ET(j,i))

!      -- Formation rate of superimposed ice (melt_star), melt rate (melt)
!         and runoff rate (runoff)

#if (ABLSURFACE==1)

      if ((beta1*ET(j,i)) <= (Pmax*snowfall(j,i))) then
         melt_star(j,i) = beta1*ET(j,i)
         melt(j,i)      = 0.0d0
         runoff(j,i)    = melt(j,i)+rainfall(j,i)
      else
         melt_star(j,i) = Pmax*snowfall(j,i)
         melt(j,i)      = beta2*(ET(j,i)-melt_star(j,i)/beta1)
         runoff(j,i)    = melt(j,i)+rainfall(j,i)
      end if

#elif (ABLSURFACE==2)

      if ( rainfall(j,i) <= (Pmax*snowfall(j,i)) ) then

         if ( (rainfall(j,i)+beta1*ET(j,i)) <= (Pmax*snowfall(j,i)) ) then
            melt_star(j,i) = rainfall(j,i)+beta1*ET(j,i)
            melt(j,i)      = 0.0d0
            runoff(j,i)    = melt(j,i)
         else
            melt_star(j,i) = Pmax*snowfall(j,i)
            melt(j,i)      = beta2 &
                             *(ET(j,i)-(melt_star(j,i)-rainfall(j,i))/beta1)
            runoff(j,i)    = melt(j,i)
         end if

      else

         melt_star(j,i) = Pmax*snowfall(j,i)
         melt(j,i)      = beta2*ET(j,i)
         runoff(j,i)    = melt(j,i) + rainfall(j,i)-Pmax*snowfall(j,i)

      end if

#endif

#elif (ABLSURFACE==3)

      temp_jja_help  = one_third*(temp_mm(j,i,6)+temp_mm(j,i,7)+temp_mm(j,i,8))

      melt_star(j,i) = 0.0d0   ! no superimposed ice considered
      melt(j,i)      = lambda_lti*max((temp_jja_help-temp_lti), 0.0d0)
      runoff(j,i)    = melt(j,i) + rainfall(j,i)

#endif

!    ---- Evaporation

      evap(j,i) = 0.0d0

!  ------ Accumulation minus ablation

      as_perp(j,i) = accum(j,i) - evap(j,i) - runoff(j,i)

!  ------ Ice-surface temperature (10-m firn temperature) temp_s,
!         including empirical firn-warming correction due to
!         refreezing meltwater when superimposed ice is formed

      if (melt_star(j,i).ge.melt(j,i)) then
         temp_s(j,i) = temp_ma(j,i) &
                       +mu*(melt_star(j,i)-melt(j,i))
      else
         temp_s(j,i) = temp_ma(j,i)
      end if

      if (temp_s(j,i) > -0.001d0) temp_s(j,i) = -0.001d0
                                  ! Cut-off of positive air temperatures

   end if

end do
end do

end subroutine boundary
