!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!   Module     :  i n s t e m p

!   Purpose    :  Computation of the daily mean surface temperature of Mars
!                 based on obliquity, eccentricity and the anomaly of vernal
!                 equinox (local insolation temperature = LIS scheme,
!                 by Bjoern Grieger, modified by Ralf Greve, 2006-01-25).

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

module instemp

   implicit none

   type ins
      double precision :: t(0:360,-90:90)
      double precision :: tam(-90:90)
      double precision :: tmax(-90:90)
   end type ins

contains

!-------- Main subroutine --------

   subroutine setinstemp ( o, ecc, ave, obl, sma, sa, op, ct )

      implicit none

      type(ins)                  :: o
      double precision, optional :: ecc   ! eccentricity
      double precision, optional :: ave   ! anom. of vernal equinox in degree
      double precision, optional :: obl   ! obliquity in degree
      double precision, optional :: sma   ! semi-major axis in AU
      double precision, optional :: sa    ! surface albedo
      double precision, optional :: op    ! orbital period in seconds
      double precision, optional :: ct    ! CO2 condensation temperature in K

      logical            :: co2layer(-90:90)
      integer            :: iphi, ipsi, year
      integer, parameter :: yearmax = 2
      double precision   :: albact, delta, du, e, eps, f, &
                            lambda, phi, psi, psi0, r
      double precision   :: tau0, teq, ufac, usum, w, wg
      double precision, dimension(-90:90) :: co2, t

      double precision :: j0
      double precision :: a
      double precision :: alb
      double precision :: u
      double precision :: tco2

      j0   = 1367.6d0   ! solar constant for Earth in W/m**2

      if ( present(ecc) ) then
         e = ecc
      else
         e = 0.0935d0
      end if
      if ( present(ave) ) then
         psi0 = ave
      else
         psi0 = 109.13d0
      end if
      if ( present(obl) ) then
         eps = obl
      else
         eps = 25.19d0
      end if
      if ( present(sma) ) then
         a = sma
      else
         a = 1.524d0
      end if
      if ( present(sa) ) then
         alb = sa
      else
         alb = 0.3d0
      end if
      if ( present(op) ) then
         u = op
      else
         u = 686.95d0*24.d0*3600.d0
      end if
      if ( present(ct) ) then
         tco2 = ct
      else
         tco2 = 146.d0
      end if

      j0 = j0 / a**2
      f = a * e
      psi0 = psi0 * acos(-1.d0)/180.d0
      eps = eps * acos(-1.d0)/180.d0

      usum = 0.0d0
      do ipsi = 0, 360, 1
         psi = ipsi * acos(-1.d0)/180.d0
         r = ( a**2 - f**2 ) / ( a + f*cos(psi) )
         du = r**2
         if (ipsi<360) usum = usum + du
      end do
      ufac = u / usum

      t = 273.15d0
      co2 = 0.0d0
      co2layer = .false.
      do year = 1, yearmax
         usum = 0.0d0
         do ipsi = 0, 360, 1
            psi = ipsi * acos(-1.d0)/180.d0
            r = ( a**2 - f**2 ) / ( a + f*cos(psi) )
            du = ufac * r**2
            if (ipsi<360) usum = usum + du
            lambda = psi - psi0
            delta = asin( sin(eps) * sin(lambda) )
            do iphi = -89, 89, 1
               phi = iphi * acos(-1.d0)/180.d0
               if ( -tan(phi) * tan(delta) .le. -1.0d0 ) then
                  tau0 = acos(-1.d0)
               else if ( -tan(phi) * tan(delta) .ge. 1.0d0 ) then
                  tau0 = 0.0d0
               else
                  tau0 = acos( -tan(phi) * tan(delta) )
               end if
               w = ( tau0*sin(phi)*sin(delta) + &
                     cos(phi)*cos(delta)*sin(tau0) ) &
                   * j0 * a**2 / ( acos(-1.d0) * r**2 )
               wg = 0.0d0 ! 0.045 * j0
               if ( t(iphi) < -999.d0 ) then
                  albact = 0.95d0
               else
                  albact = alb
               end if
               teq = ( ( wg + (1.d0-albact) * w ) / 5.67051d-8 )**.25d0
               if ( teq < tco2 ) then
                  co2layer(iphi) = .true.
                  co2(iphi) = co2(iphi) + 5.67051d-8 * tco2**4 * du &
                              - (1.d0-albact) * w * du
                  t(iphi) = tco2
               else
                  if ( co2layer(iphi) ) then
                     co2(iphi) = co2(iphi) + 5.67051d-8 * tco2**4 * du &
                                 - (1.d0-albact) * w * du
                     if ( co2(iphi) .le. 0.0d0 ) then
                        co2(iphi) = 0.0d0
                        co2layer(iphi) = .false.
                        t(iphi) = tco2
                     end if
                  else
                     t(iphi) = teq
                  end if
               end if
               if ( year .eq. yearmax ) o%t(ipsi,iphi) = t(iphi)
            end do
            if ( year .eq. yearmax .and. ipsi .eq. 0 ) then
               o%tam = 0.0d0
               o%tmax = 0.0d0
            else
               o%tam = o%tam + t
               do iphi = -89, 89
                  o%tmax(iphi) = max( o%tmax(iphi), t(iphi) )
               end do
            end if
         end do
         o%tam = o%tam / 360.d0
      end do
      o%t(:,-90)  = o%t(:,-89)  + ( o%t(:,-89)  - o%t(:,-88) )  / 2.d0
      o%t(:, 90)  = o%t(:, 89)  + ( o%t(:, 89)  - o%t(:, 88) )  / 2.d0
      o%tam(-90)  = o%tam(-89)  + ( o%tam(-89)  - o%tam(-88) )  / 2.d0
      o%tam( 90)  = o%tam( 89)  + ( o%tam( 89)  - o%tam( 88) )  / 2.d0
      o%tmax(-90) = o%tmax(-89) + ( o%tmax(-89) - o%tmax(-88) ) / 2.d0
      o%tmax( 90) = o%tmax( 89) + ( o%tmax( 89) - o%tmax( 88) ) / 2.d0
   end subroutine
   
!-------- Annual mean temperature at latitude phi --------

   double precision function instam ( o, phi )

      implicit none
      type(ins)        :: o
      double precision :: phi
      integer          :: iphi1, iphi2

      iphi1 = nint( phi - 0.5d0 )
      if ( iphi1 < -90 ) then
         iphi1 = -90
      else if ( iphi1 > 89 ) then
         iphi1 = 89
      end if
      iphi2 = iphi1 + 1
      instam = o%tam(iphi1) + ( o%tam(iphi2) - o%tam(iphi1) ) &
               * ( phi - iphi1 )

   end function instam

!-------- Annual maximum temperature at latitude phi --------

   double precision function instmax ( o, phi )

      implicit none
      type(ins)        :: o
      double precision :: phi
      integer          :: iphi1, iphi2

      iphi1 = nint( phi - 0.5d0 )
      if ( iphi1 < -90 ) then
         iphi1 = -90
      else if ( iphi1 > 89 ) then
         iphi1 = 89
      end if
      iphi2 = iphi1 + 1
      instmax = o%tmax(iphi1) + ( o%tmax(iphi2) - o%tmax(iphi1) ) &
               * ( phi - iphi1 )

   end function instmax

!-------- Temperature at orbit position psi and latitude phi --------

   double precision function inst ( o, psi, phi )

      implicit none
      type(ins)        :: o
      double precision :: psi, phi
      integer          :: ipsi1, ipsi2

      ipsi1 = nint( psi - 0.5d0 )
      if ( ipsi1 < 0 ) then
         ipsi1 = 0
      else if ( ipsi1 > 359 ) then
         ipsi1 = 359
      end if
      ipsi2 = ipsi1 + 1
      inst = inst1(o,ipsi1,phi) + ( inst1(o,ipsi2,phi) - inst1(o,ipsi1,phi) ) &
               * ( psi - ipsi1 )

   end function inst

   double precision function inst1 ( o, ipsi, phi )

      implicit none
      type(ins)        :: o
      integer          :: ipsi
      double precision :: phi
      integer          :: iphi1, iphi2

      iphi1 = nint( phi - 0.5d0 )
      if ( iphi1 < -90 ) then
         iphi1 = -90
      else if ( iphi1 > 89 ) then
         iphi1 = 89
      end if
      iphi2 = iphi1 + 1
      inst1 = o%t(ipsi,iphi1) + ( o%t(ipsi,iphi2) - o%t(ipsi,iphi1) ) &
               * ( phi - iphi1 )

   end function inst1

end module instemp
